#!/usr/bin/env python
#-*- encoding: utf-8 -*-

#
# Script for extracting trees out of AVMs. To be executed after 
# *deleted-daughter-features* has been set to `nil', a sentence is parsed,
# and the resulting AVMs are written to '/tmp/output.dag':
#
# (setf lkb::*deleted-daughter-features* nil)
# 
# (lkb::do-parse-tty "הילד לומד ב הכיתה.")
#
# (with-open-file (stream "/tmp/out.dag" :direction :output
#                          :if-exists :supersede)
# 	    (loop
# 		for parse in lkb::*parse-record*
# 		do  (lkb::display-dag1 (lkb::tdfs-indef (lkb::edge-dag parse)) 'lkb::compact stream)))
#
# The script is executed with the following command:
#
# python avm2consttree.py
#
# This produces a bracketed representation. In order to produce latex
# trees, add the opional argument 'tex':
#
# python avm2consttree.py tex
# 
# Petter Haugereid, petterha@gmail.com
# 

import sys,os

heb = False

try:
    exported = open('/tmp/out.dag')
except:
    print 'AVM missing in \'/tmp/out.dag\'.' 
    print 'Do the following commands in the *common-lisp* buffer:'
    print '(setf lkb::*deleted-daughter-features* nil)'
    print '(lkb::do-parse-tty "INPUT SENTENCE")'
    print '(with-open-file (stream "/tmp/out.dag" :direction :output'
    print '              :if-exists :supersede)'
    print ' (loop'
    print '     for parse in lkb::*parse-record*'
    print '     do  (lkb::display-dag1 (lkb::tdfs-indef (lkb::edge-dag parse)) \'lkb::compact stream)))'
    quit()

try:
    texopt = sys.argv[1]
    if texopt == 'tex':
        tex = True
    else:
        tex = False
except:
    tex = False

if tex:
    texout = open('const-tree.tex','w')
    texout.write('\documentclass{article}\n')
    texout.write('\usepackage{fullpage}\n')
    texout.write('\usepackage[margin=2cm,a2paper]{geometry}\n')
    if heb:
        texout.write('\usepackage{cjhebrew}\n')
    texout.write('\usepackage{qtree}\n')
    texout.write('\pagestyle{empty}\n')
    texout.write('\\begin{document}\n')

def path2string(inlist):
    pathstring = ''
    for feature in inlist:
        pathstring = pathstring + '$' + feature
    return pathstring

avms = []
newlines = []
for line in exported:
    newlines.append(line)
    if ']main' in line:
        avms.append(newlines)
        newline = 'main' + line.split(']main')[1]
        newlines = [newline]
avms.append(newlines)

prevline = newlines[0]
def checkstack(lasttag,stacklist):
    values = tag2value[lasttag]
    for value in values:
        if value[0] == 'FIRST':
            lasttag = value[1]
            values = tag2value[lasttag]
            for value in values:
                if value[0] == 'LOCAL':
                    lasttag = value[1]
                    values = tag2value[lasttag]
                    for value in values:
                        if value[0] == 'CAT':
                            lasttag = value[1]
                            values = tag2value[lasttag]
                            for value in values:
                                if value[0] == 'HEAD':
                                    lasttag = value[1]
                                    stacklist.append(lasttag)
                            for value in values:
                                if value[0] == 'STACK':
                                    lasttag = value[1]
                                    return [lasttag,stacklist]

def path2tag(path):
    tag = '$0'
    for feature in path:
        for value in tag2value[tag]:
            possfeat = value[0]
            if possfeat == feature:
                tag = value[1]
                mytype = tag2type[tag]
                lasttag = tag
                lastfeat = feature
    if lastfeat == path[-1]:
        return lasttag
    else:
        return ''

def path2type(path):
    return tag2type[path2tag(path)]

type2word = {'aux': 'Aux', 'prep': 'P', 'prep-min': 'P', 'noun': 'N', 'noun-pn': 'N', 'verb': 'V','pron': 'Pron','cop-pron': 'Pron', 'proper-noun': 'PN', 'det': 'Det', 'card': 'Det', 'adj': 'Adj', 'mod-mod': 'Adv', 'sadv': 'SA', 'complementizer': 'C', 'subcompl': 'C', 'subcompl-omcompl': 'C', 'adv': 'Adv', 'condcompl': 'C', 'subcompl': 'C', 'relcompl': 'R', 'infcompl': 'I', 'conj': 'Conj', 'start': 'S'} 
type2phrase = {'aux': 'AuxP', 'prep': 'PP', 'prep-min': 'PP', 'noun': 'NP', 'noun-pn': 'NP', 'pron': 'NP', 'det': 'NP', 'card': 'NP', 'proper-noun': 'NP', 'adj': 'AP', 'sadv': 'AdvP', 'complementizer': 'CP', 'infcompl': 'IP', 'subcompl': 'CP', 'condcompl': 'CP', 'subcompl-omcompl': 'CP', 'relcompl': 'RP', 'adv': 'AdvP', 'verb': 'VP', 'start': 'S'} 

for avm in avms:
    parsetree = ''
    avmin = False
    spaces = 0
    arglevel = 0
    signembs = []
    orthin = False
    orthnr = 0
    stackin = False
    path = []
    pathtypes = []
    orth2rootpath = {}
    tag2type = {}
    orth = ''
    orthlist = []
    stackvalue = ''
    orthpath = ''
    path2pathtype = {}
    tags = ['$0']
    tag2value = {}
    tag2type = {}
    tag2tag = {}
    x = 1
    for line in avm[1:]:
        items = line.lstrip().split()
        feature = items[0]
        if items[1][0] == '#':
            tag = items[1]
            if tag[-1] == ',':
                tag = tag[:-1]
            if len(items) > 3 and not items[3] == ']':
                mytype = items[3]
                if mytype[-1] == ',':
                    mytype = mytype[:-1]
                tag2type[tag] = mytype
        else:
            tag = '$'+str(x)
            x = x+1
            mytype = items[1]
            if mytype[-1] == ',':
                mytype = mytype[:-1]
            tag2type[tag] = mytype

        if not '[' in prevline:
            path.pop()
            pathtypes.pop()
            if ']' in prevline:
                previtems = prevline.lstrip().split()
                for item in previtems:
                    if ']' in item:
                        path.pop()
                        pathtypes.pop()
                        tags.pop()
                tags.pop()
                tags.append(tag)
            else:
                tags.pop()
                tags.append(tag)
        else:
            tags.append(tag)
        mothertag = tags[-2]
        tag2value[mothertag] = tag2value.get(mothertag, []) + [[feature, tag]]
        path.append(feature)
        pathtypes.append(feature+'@'+tag+'@'+mytype)
        emb = len(path)
        if 'ARGS cons & [\n' in line:
            arglevel = arglevel+1
            signemb = emb
            signembs.append(signemb)
            parsetree = parsetree + '['
        try:
            if 'REST cons & [\n' in line and signembs[-1]+1 == emb:
                parsetree = parsetree + ']'
                parsetree = parsetree + '['
        except:
            pass
        if len(signembs)>0:
         if 'REST null ]' in line and signembs[-1]-1 == emb:
            parsetree = parsetree + ']'
            signembs.pop()
            arglevel = arglevel-1
        if 'STEM ' in line and 'orthography & [' in line:
            orthin = True
        if 'REST null' in line and orthin:
            orthin = False
        if orthin and 'FIRST' in line:
            items = line.split()
            orth = items[-1]
            if orth[0] == '"':
                orth = orth + '_' + str(orthnr)
                orthnr = orthnr+1
                orthlist.append(orth)
                orth2rootpath[orth] = path[:]
            parsetree = parsetree + orth
            
        if len(path) > 3:
            if path[-4:] == ['SYNSEM', 'LOCAL', 'CAT', 'STACK'] and not 'STACK' in path[:-4]:
                stackin = True
                stackemb = len(path)

        if stackin:
            if not 'STACK' in path:
                stackin = False

        prevline = line

    lastpath = ['ARGS','FIRST']
    while not mytype == 'start-word':
        lastpath = lastpath + ['ARGS','FIRST']
        tag = path2tag(lastpath)
        mytype = tag2type[tag]
    
    def listlen(path):
        tag = path2tag(path)
        mytype = tag2type[tag]
        x=0
        while not mytype == 'null':
            path = path + ['REST']
            tag = path2tag(path)
            mytype = tag2type[tag]
            x=x+1
        return x
    
    
    olddepth = 0
    oldslash = 'null'
    if tex:
        stacktree = '\\Tree [.S '
    else:
        stacktree = '[ S '
    tag2ind = {}
    indices = ['n','m','l','k','j','i']
    elist = set(['null','0-1-list',''])
    nelist = set(['cons','1-list'])
    while len(lastpath) > 0:
        tag = path2tag(lastpath)
        mytype = tag2type[tag]
        stackpath = lastpath + ['SYNSEM','LOCAL','CAT','STACK']
        stacktype = path2type(stackpath)
        mstackpath = lastpath[2:] + ['SYNSEM','LOCAL','CAT','STACK']
        mstacktype = path2type(mstackpath)
        newdepth = listlen(mstackpath)
        slashpath = lastpath + ['SYNSEM','NON-LOCAL','SLASH']
        try:
            slashtype = path2type(slashpath)
            slashtag = path2tag(slashpath + ['FIRST','CAT','HEAD'])
        except:
            slashtype = ''
            slashtag = ''
        mslashpath =  lastpath[:-2] + ['SYNSEM','NON-LOCAL','SLASH']
        mslashtag = path2tag(mslashpath)
        mslashtype = path2type(mslashpath)
        vblpath = lastpath + ['SYNSEM', 'LOCAL', 'CAT', 'VBL']
        vbltype = path2type(vblpath)
        mvblpath = lastpath[:-2] + ['SYNSEM', 'LOCAL', 'CAT', 'VBL']
        mvbltype = path2type(mvblpath)
        headpath = lastpath[:-2] + ['SYNSEM', 'LOCAL', 'CAT', 'HEAD']
        headtag = path2tag(headpath)
        indpath = lastpath + ['SYNSEM', 'LOCAL', 'CONT', 'HOOK','INDEX']
        indtag = path2tag(indpath)
        mindpath = lastpath[:-2] + ['SYNSEM', 'LOCAL', 'CONT', 'HOOK','INDEX']
        mindtag = path2tag(mindpath)
        try:
            mheadtag = path2tag(headpath[2:])
        except:
            mheadtag = headtag
        headtype = path2type(headpath)
        argrestpath = lastpath[:-1] + ['REST']
        argresttype = path2type(argrestpath)
        try:
            phrkey = path2type(lastpath[2:] + ['SYNSEM','--PHR-KEY'])
        except:
            phrkey = 'relation'
        if 'arg' in phrkey:
            if tex:
                arg = '{\\footnotesize ' + phrkey[3] + '}'
            else:
                arg = '_' + phrkey[3]
        else:
            arg = ''
        index = ''
        if stacktype in elist and mstacktype in nelist:
            mstackslashpath = mstackpath + ['FIRST','NON-LOCAL','SLASH']
            try:
                mstackslashtype = path2type(mstackslashpath)
            except:
                mstackslashtype = '0-1-list'
            if (mstackslashtype in nelist and slashtype in elist) or (mslashtype in nelist and slashtype in elist and mstackslashtype in elist):
                slashtag = mindtag
                tag = slashtag[1:]
                if not tag in tag2ind.keys():
                    tag2ind[tag] = indices.pop()
                if tex:
                    index = '_{' + tag2ind[tag]+ '}'
                else:
                    index = '_' + tag2ind[tag]
        try:
            phrase = type2phrase[headtype] + arg
        except:
            phrase = '?'
        if newdepth > olddepth:
            x = newdepth - olddepth
            while x > 0:
                if tex:
                    stacktree = stacktree + '[.'
                else:
                    stacktree = stacktree + '[ '
                phrase = phrase + index
                stacktree = stacktree + phrase + ' '
                x=x-1
        elif newdepth < olddepth:
            x = olddepth - newdepth
            while x > 0:
                stacktree = stacktree + '] '
                x=x-1
        elif argresttype in elist and mslashtype in elist and slashtype in nelist:
            tag = path2tag(slashpath + ['FIRST','CAT','HEAD'])
            slashhead = tag2type[tag]
            slashindtag = path2tag(slashpath + ['FIRST','CONT','HOOK','INDEX'])
            tag = slashindtag[1:]
            try:
                if tex:
                    index = '_{' + tag2ind[tag]+ '} '
                else:
                    index = '_' + tag2ind[tag]+ ' '
            except:
                if tex:
                    index = '_{j} '
                else:
                    index = '_j '

            phrase = type2phrase[slashhead] + arg + index
            stacktree = stacktree + phrase
        elif argresttype == 'null' and vbltype == 'synsem' and mvbltype == 'anti-synsem':
            word = type2word['pron']
            stacktree = stacktree + word + ' '
        # Words
        if not argresttype == 'null':
            head = path2type(argrestpath + ['FIRST','SYNSEM','LOCAL','CAT','HEAD'])
            try:
                word = type2word[head]
            except:
                word = '?'
            if tex:
                stacktree = stacktree + '[.' + word + ' '
            else:
                stacktree = stacktree + '[ ' + word + ' '
            orth = path2type(argrestpath + ['FIRST','STEM','FIRST'])
            z = 0
            if heb:
                if word == 'V':
                    binyan = path2type(argrestpath + ['FIRST','ARGS','FIRST','INFLECTION'])
                if word == 'N':
                    gender = path2type(argrestpath + ['FIRST','SYNSEM','LOCAL','CONT','HOOK','INDEX','PNG','GEN'])
            while not orth[0] == '"':
                argrestpath = argrestpath + ['FIRST','ARGS']
                head = path2type(argrestpath + ['FIRST','SYNSEM','LOCAL','CAT','HEAD'])
                z = z+1
                orth = path2type(argrestpath + ['FIRST','STEM','FIRST'])
            if heb:
                from hebreverse import hebreverse
                orth = hebreverse(orth)
                from heb2tex import heb2tex
                orth = heb2tex(orth)
            if orth[0] == '"':
                orth = orth[1:-1]
            orth = orth
            if tex:
                if heb:
                    orth = '{\\<' + orth + '>}'
                    if word == 'V' and not binyan == 'inflection':
                        orth = '_{' + binyan[1] + '}' + orth
                    if word == 'N':
                        orth =  '_{' + gender[0] + '}' + orth
            stacktree = stacktree + orth  + ' ] '
        lastpath = lastpath[2:]
        olddepth = newdepth
        oldslash = slashtype
    stacktree = stacktree + ']'
    if tex:
        texout.write(stacktree + '\n\n\\bigskip\n')
    else:
        print stacktree

if tex:
    texout.write('\end{document}\n')
    texout.close()
    os.system('latex const-tree.tex\n')
    os.system('dvips -o const-tree.ps const-tree\n')
    os.system('ps2eps -f const-tree.ps\n')
    os.system('evince const-tree.eps & \n')
